from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="cavora",
    version="3.0.0",
    author="Samir Baladi",
    author_email="gitdeeper@gmail.com",
    description="CAVORA - Cave Passage Stability and Hydrology Framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/gitdeeper4/cavova",
    project_urls={
        "Documentation": "https://cavora.netlify.app",
        "Source Code": "https://gitlab.com/gitdeeper4/cavova",
        "Bug Tracker": "https://gitlab.com/gitdeeper4/cavova/-/issues",
    },
    license="CC BY 4.0",
    packages=find_packages(exclude=["tests", "scripts", "notebooks", "docs"]),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Hydrology",
        "License :: Other/Proprietary License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    python_requires=">=3.8",
    install_requires=[
        "numpy>=1.19.0",
        "scipy>=1.5.0",
        "pandas>=1.1.0",
        "matplotlib>=3.3.0",
        "plotly>=5.0.0",
        "dash>=2.0.0",
    ],
    extras_require={
        "dev": ["pytest>=6.0.0", "black>=21.0.0", "flake8>=3.9.0", "mypy>=0.900"],
        "plot": ["matplotlib>=3.3.0", "plotly>=5.0.0"],
    },
    entry_points={
        "console_scripts": [
            "cavora-stability=scripts.generate_stability_report:main",
            "cavora-flood=scripts.predict_flood_pulse:main",
            "cavora-cdz=scripts.map_cdz_zones:main",
        ],
    },
    include_package_data=True,
    zip_safe=False,
)
